@@ -1,4 +1,4 @@
-poisson.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.tau2=NULL, fix.rho.slo=FALSE, rho.slo=NULL, fix.rho.int=FALSE, rho.int=NULL, MALA=TRUE, verbose=TRUE)
+poisson.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.tau2=NULL, rho.slo=NULL, rho.int=NULL, MALA=TRUE, verbose=TRUE)
 {
 #### Verbose
 a <- common.verbose(verbose)  
@@ -15,9 +15,9 @@ X.mean <- frame.results$X.mean
 X.indicator <- frame.results$X.indicator 
 offset <- frame.results$offset
 Y <- frame.results$Y
-Y.miss <- frame.results$Y.miss
 which.miss <- frame.results$which.miss
 n.miss <- frame.results$n.miss  
+Y.DA <- Y  
 
 
 #### Check on MALA argument
@@ -26,36 +26,35 @@ n.miss <- frame.results$n.miss
 
 
 #### Check on the rho arguments
-    if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
-    if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
-    if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
-    if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
-    if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
-    if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)  
-    if(fix.rho.int)
+    if(is.null(rho.int))
+    {
+    rho <- runif(1)
+    fix.rho.int <- FALSE   
+    }else
     {
     rho <- rho.int
-    }else
-    {
-    rho <- runif(1)       
+    fix.rho.int <- TRUE
     }
-    if(fix.rho.slo)
+    if(!is.numeric(rho)) stop("rho.int is fixed but is not numeric.", call.=FALSE)  
+    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)    
+
+    if(is.null(rho.slo))
+    {
+    lambda <- runif(1)
+    fix.rho.slo <- FALSE   
+    }else
     {
     lambda <- rho.slo
-    }else
-    {
-    lambda <- runif(1)       
-    }   
-    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
-    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    fix.rho.slo <- TRUE
+    }
+    if(!is.numeric(lambda)) stop("rho.slo is fixed but is not numeric.", call.=FALSE)  
     if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
     if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
 
-    
+
 #### CAR quantities
-rho.sum <- rho + lambda
-fix.rho.sum <- as.logical(fix.rho.int * fix.rho.slo)
-W.quants <- common.Wcheckformat.leroux(W, fix.rho.sum, rho.sum)
+W.quants <- common.Wcheckformat.leroux(W)
 K <- W.quants$n
 N <- N.all / K
 W <- W.quants$W
@@ -68,10 +67,10 @@ W.begfin <- W.quants$W.begfin
 
 #### Priors
     if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(100000, p)
     if(is.null(prior.tau2)) prior.tau2 <- c(1, 0.01)
     if(is.null(prior.mean.alpha)) prior.mean.alpha <- rep(0, 1)
-    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(1000, 1)
+    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(100000, 1)
 prior.beta.check(prior.mean.beta, prior.var.beta, p)
 prior.var.check(prior.tau2)
     if(length(prior.mean.alpha)!=1) stop("the prior mean for alpha is the wrong length.", call.=FALSE)    
@@ -123,6 +122,15 @@ tau2.phi <- var(phi)/10
 tau2.delta <- var(delta)/10
 
 
+#### Specify matrix quantities
+offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
+regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
+time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
+delta.time.mat <- apply(time.mat, 2, "*", delta)
+phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)   
+lp <- as.numeric(offset.mat + regression.mat + phi.mat + delta.time.mat + alpha * time.mat)
+fitted <- exp(lp)
+
 
 ###############################    
 #### Set up the MCMC quantities    
@@ -138,8 +146,7 @@ samples.delta <- array(NA, c(n.keep, K))
 samples.tau2 <- array(NA, c(n.keep, 2))
 colnames(samples.tau2) <- c("tau2.int", "tau2.slo")
 samples.fitted <- array(NA, c(n.keep, N.all))
-samples.like <- array(NA, c(n.keep, N.all))
-samples.deviance <- array(NA, c(n.keep, 1))
+samples.loglike <- array(NA, c(n.keep, N.all))
     if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
     
@@ -174,20 +181,6 @@ tau2.delta.shape <- prior.tau2[1] + K/2
     if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
 
 
-
-##########################################
-#### Specify quantities that do not change
-########################################## 
-offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
-regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
-Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
-Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
-phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
-time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
-delta.time.mat <- apply(time.mat, 2, "*", delta)
-which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
-
-
 #### Check for islands
 W.list<- mat2listw(W)
 W.nb <- W.list$neighbours
@@ -205,7 +198,7 @@ n.islands <- max(W.islands$nc)
 ## Start timer
     if(verbose)
     {
-    cat("Generating", n.keep, "post burnin and thinned (if requested) samples\n", sep = " ")
+    cat("Generating", n.keep, "post burnin and thinned (if requested) samples.\n", sep = " ")
     progressBar <- txtProgressBar(style = 3)
     percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -217,16 +210,28 @@ n.islands <- max(W.islands$nc)
 #### Create the MCMC samples    
     for(j in 1:n.sample)
     {
+    ####################################
+    ## Sample from Y - data augmentation
+    ####################################
+        if(n.miss>0)
+        {
+        Y.DA[which.miss==0] <- rpois(n=n.miss, lambda=fitted[which.miss==0])    
+        }else
+        {}
+    Y.DA.mat <- matrix(Y.DA, nrow=K, ncol=N, byrow=FALSE)
+    
+        
+        
     ####################
     ## Sample from beta
     ####################
     offset.temp <- offset + as.numeric(phi.mat) + as.numeric(delta.time.mat) + as.numeric(alpha * time.mat)      
         if(p>2)
         {
-        temp <- poissonbetaupdateMALA(X.standardised, N.all, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, n.beta.block, proposal.sd.beta, list.block)
+        temp <- poissonbetaupdateMALA(X.standardised, N.all, p, beta, offset.temp, Y.DA, prior.mean.beta, prior.var.beta, n.beta.block, proposal.sd.beta, list.block)
         }else
         {
-        temp <- poissonbetaupdateRW(X.standardised, N.all, p, beta, offset.temp, Y.miss, prior.mean.beta, prior.var.beta, which.miss, proposal.sd.beta)
+        temp <- poissonbetaupdateRW(X.standardised, N.all, p, beta, offset.temp, Y.DA, prior.mean.beta, prior.var.beta, proposal.sd.beta)
         }
     beta <- temp[[1]]
     accept[1] <- accept[1] + temp[[2]]
@@ -242,8 +247,8 @@ n.islands <- max(W.islands$nc)
     prob1 <- 0.5 * (alpha - prior.mean.alpha)^2 / prior.var.alpha - 0.5 * (proposal.alpha - prior.mean.alpha)^2 / prior.var.alpha
     lp.current <- offset + as.numeric(regression.mat) + as.numeric(phi.mat) + as.numeric(delta.time.mat) + as.numeric(alpha * time.mat)     
     lp.proposal <- offset + as.numeric(regression.mat) + as.numeric(phi.mat) + as.numeric(delta.time.mat) + as.numeric(proposal.alpha * time.mat)            
-    like.current <- Y * lp.current - exp(lp.current)
-    like.proposal <- Y * lp.proposal - exp(lp.proposal)
+    like.current <- Y.DA * lp.current - exp(lp.current)
+    like.proposal <- Y.DA * lp.proposal - exp(lp.proposal)
     prob2 <- sum(like.proposal - like.current, na.rm=TRUE)
     prob <- exp(prob1 + prob2)
         if(prob > runif(1))
@@ -263,10 +268,10 @@ n.islands <- max(W.islands$nc)
     phi.offset <- offset.mat + regression.mat + delta.time.mat + alpha * time.mat
         if(MALA)
         {
-        temp1 <- poissoncarupdateMALA(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, Y.mat.miss, proposal.sd.phi, rho, phi.offset, N, rep(1,N), which.miss.mat)
+        temp1 <- poissoncarupdateMALA(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, Y.DA.mat, proposal.sd.phi, rho, phi.offset, N, rep(1,N))
         }else
         {
-        temp1 <- poissoncarupdateRW(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, Y.mat.miss, proposal.sd.phi, rho, phi.offset, N, rep(1,N), which.miss.mat)
+        temp1 <- poissoncarupdateRW(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, Y.DA.mat, proposal.sd.phi, rho, phi.offset, N, rep(1,N))
         }
     phi <- temp1[[1]]
         if(rho<1)
@@ -288,10 +293,10 @@ n.islands <- max(W.islands$nc)
     delta.offset <- offset.mat + regression.mat + phi.mat +  alpha * time.mat
         if(MALA)
         {
-        temp2 <- poissoncarupdateMALA(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.mat.miss, proposal.sd.delta, lambda, delta.offset, N, time, which.miss.mat)
+        temp2 <- poissoncarupdateMALA(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.DA.mat, proposal.sd.delta, lambda, delta.offset, N, time)
         }else
         {
-        temp2 <- poissoncarupdateRW(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.mat.miss, proposal.sd.delta, lambda, delta.offset, N, time, which.miss.mat)
+        temp2 <- poissoncarupdateRW(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.DA.mat, proposal.sd.delta, lambda, delta.offset, N, time)
         }
     delta <- temp2[[1]]
         if(lambda <1)
@@ -382,10 +387,7 @@ n.islands <- max(W.islands$nc)
     #########################
     lp <- as.numeric(offset.mat + regression.mat + phi.mat + delta.time.mat + alpha * time.mat)
     fitted <- exp(lp)
-    deviance.all <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
-    like <- exp(deviance.all)
-    deviance <- -2 * sum(deviance.all, na.rm=TRUE)    
-        
+    loglike <- dpois(x=as.numeric(Y), lambda=fitted, log=TRUE)
 
     
     ###################
@@ -401,10 +403,9 @@ n.islands <- max(W.islands$nc)
             if(!fix.rho.int) samples.rho[ele, ] <- rho
             if(!fix.rho.slo) samples.lambda[ele, ] <- lambda
         samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
-        samples.deviance[ele, ] <- deviance
         samples.fitted[ele, ] <- fitted
-        samples.like[ele, ] <- like
-            if(n.miss>0) samples.Y[ele, ] <- rpois(n=n.miss, lambda=fitted[which.miss==0])
+        samples.loglike[ele, ] <- loglike
+            if(n.miss>0) samples.Y[ele, ] <- Y.DA[which.miss==0]
         }else
         {}
         
@@ -449,7 +450,7 @@ n.islands <- max(W.islands$nc)
 #### end timer
     if(verbose)
     {
-    cat("\nSummarising results")
+    cat("\nSummarising results.")
     close(progressBar)
     }else
     {}
@@ -482,44 +483,30 @@ accept.final <- c(accept.beta, accept.alpha, accept.phi, accept.delta, accept.rh
 names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho.int", "rho.slo")
    
 
-#### Deviance information criterion (DIC)
-median.phi <- apply(samples.phi, 2, median)
-median.delta <- apply(samples.delta, 2, median)
-median.alpha <- median(samples.alpha)
-median.phi.mat <- matrix(rep(median.phi, N), byrow=F, nrow=K)
-delta.time.mat <- apply(time.mat, 2, "*", median.delta)
-median.beta <- apply(samples.beta,2,median)
-regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=FALSE)   
-lp.median <- offset.mat + regression.mat + phi.mat + delta.time.mat + median.alpha * time.mat
-fitted.median <- exp(lp.median)
-deviance.fitted <- -2 * sum(dpois(x=as.numeric(Y), lambda=fitted.median, log=TRUE), na.rm=TRUE)
-p.d <- median(samples.deviance) - deviance.fitted
-DIC <- 2 * median(samples.deviance) - deviance.fitted  
+#### Compute the fitted deviance
+mean.phi <- apply(samples.phi, 2, mean)
+mean.delta <- apply(samples.delta, 2, mean)
+mean.alpha <- mean(samples.alpha)
+mean.phi.mat <- matrix(rep(mean.phi, N), byrow=F, nrow=K)
+delta.time.mat <- apply(time.mat, 2, "*", mean.delta)
+mean.beta <- apply(samples.beta,2,mean)
+regression.mat <- matrix(X.standardised %*% mean.beta, nrow=K, ncol=N, byrow=FALSE)   
+lp.mean <- offset.mat + regression.mat + mean.phi.mat + delta.time.mat + mean.alpha * time.mat
+fitted.mean <- exp(lp.mean)
+deviance.fitted <- -2 * sum(dpois(x=as.numeric(Y), lambda=fitted.mean, log=TRUE), na.rm=TRUE)
+
+   
+#### Model fit criteria
+modelfit <- common.modelfit(samples.loglike, deviance.fitted)
 
 
-#### Watanabe-Akaike Information Criterion (WAIC)
-LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
-p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
-WAIC <- -2 * (LPPD - p.w)
-
-
-## Compute the LMPL
-CPO <- rep(NA, N.all)
-    for(j in 1:N.all)
-    {
-    CPO[j] <- 1/median((1 / dpois(x=Y[j], lambda=samples.fitted[ ,j])))    
-    }
-LMPL <- sum(log(CPO), na.rm=TRUE)  
-    
-    
-## Create the fitted values and residuals
-fitted.values <- apply(samples.fitted, 2, median)
+#### Create the fitted values and residuals
+fitted.values <- apply(samples.fitted, 2, mean)
 response.residuals <- as.numeric(Y) - fitted.values
 pearson.residuals <- response.residuals /sqrt(fitted.values)
-deviance.residuals <- sign(response.residuals) * sqrt(2 * (Y * log(Y/fitted.values) + fitted.values - Y))
-residuals <- data.frame(response=response.residuals, pearson=pearson.residuals, deviance=deviance.residuals)
-
+residuals <- data.frame(response=response.residuals, pearson=pearson.residuals)
     
+
 #### Transform the parameters back to the origianl covariate scale.
 samples.beta.orig <- common.betatransform(samples.beta, X.indicator, X.mean, X.sd, p, FALSE)  
 
@@ -565,11 +552,6 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
 ## Compile and return the results
-loglike <- (-0.5 * deviance.fitted)
-modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike)
-names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood")
-
-
 #### Harmonise samples in case of them not being generated
     if(fix.rho.int & fix.rho.slo)
     {
@@ -598,7 +580,7 @@ class(results) <- "CARBayesST"
     if(verbose)
     {
     b<-proc.time()
-    cat(" finished in ", round(b[3]-a[3], 1), "seconds")
+    cat("Finished in ", round(b[3]-a[3], 1), "seconds.\n")
     }else
     {}
 return(results)
